function algebra(desired_feature_grid,desired_duality)
% CONCEPT_ALGEBRA package v0.1 by Jacob Feldman (4/2004)
% 
% This package computes the elements of the concept algebra, including
% the implication polynomial power series, power spectrum, and algebraic
% complexity, for a given user-entered concept with up to 10 discrete-valued
% features, each of which can take an arbitrary number of values.
% Please see Feldman (2001) "An algebra of human concepts" for full definitions
% and explanations.
% 
% Principal functions in the package:
%  
% COMPLEXITY_TOOL: Graphical user interface for entering and evaluating concepts.
%   This is the recommended front-end for concepts with D <= 5. It allows
%   graphical access to most of the functions listed below. For D > 5, use
%   the following functions from the command line.
%
% Command-line functions:  [see below for explanation of syntax]
%
% POWER_SERIES(concept):  computes the implication power series of concept.
%
%     Examples: 
%          >> make_feature_grid([3 3 3]);              % initializes 3 by 3 by 3 feature_grid
%          >> concept = [0 0 0; 1 2 0];                % defines concept with two objects
%          >> series = power_series(concept,'ruleout') % note: second argument is optional
%
% POWER_SPECTRUM(concept):      computes the power spectrum of a concept
% SPECTRUM_OF_SERIES(series)    computes the power spectrum given a series
%
%     Examples: 
%          >> spectrum = power_spectrum(concept)
%          >> spectrum = spectrum_of_series(power_series(concept))
%          >> bar(0:size(concept,2)-1,spectrum);  % makes bar plot of spectrum
%
% COMPLEXITY(concept):              computes the algebraic complexity of a concept
% COMPLEXITY_OF_SERIES(series):     computes the algebraic complexity given a series
%
%     Examples:
%          >> complexity(concept)
%          >> complexity_of_series(power_series(concept))
%
% ALGEBRA(feature_grid,duality) initializes the algebra package and/or sets
%    values for global variables feature_grid and duality
%  
%   
%
%
% Syntax and terminology: 
% 
% The "feature grid" is a 1-by-D vector in which each entry
% gives the number of feature values allowed at that position.
%
% An "object" is a 1-by-D vector of feature values, with
% each feature taking values from 0 to V-1, where V is the entry
% at that point in (global) feature_grid. 
%     Example: [0 3 2] is a valid object chosen from feature_grid [4 4 4]
%        but [0 0 0 0] is not (too many features) and neither is [0 4 0], 
%        (second feature out of bounds). 
% A "concept" is a P-by-D matrix consisting of P objects of dimension D.
%     Example: [0 3 2; 0 0 0] is a valid concept consisting of 2 objects of
%     dimension 3. 
%
% An "implication polynomial" or "regularity" is a 1-by-D+1 vector, where the
% first D entries are either values or -1, which means "don't care". The D+1-th
% entry is  the degree of the concept (the total number of non-negative entries). 
%     Example: [2 -1 0 2] is a regularity of dimension 3, degree 2. 
%  Note that each regularity describes an area of the grid. The interpretation
%  of this area in the theory depends on the duality; see below.
%
% The "duality" is a global variable taking values 'ruleout' or 'rulein'
% (equivalent to conjunctive and disjunctive duality, respectively, as
% described in Appendix II of Feldman, 2001). 
%
% A "power series" or "theory" is an N-by-D+1 matrix of regularities. The "model" of a
% power series (e.g. as provided by private_theory2model) is the set
% of objects (i.e. the concept) described by that series. In ruleout duality,
% this means the entire area of the feature grid not indicated (ruled out) by any
% of the regularities. In rulein duality, this means the entire area of the
% feature grid indicated (ruled in) by any of the regularities. In ruleout
% duality, the regularities union to form the prohibited area (or, equivalently,
% the regularities taken as ruleouts intersect to form the allowed area);
% in rulein duality, the regularities union to form the concept (i.e. the allowed area). 
%
% The power series for a given concept is the minimal power series whose model is
% the concept; see Feldman (2001), Appendix, for proof and more careful definitions. 
%
% The "spectrum" of a concept a 1-by-D vector of integers; the i-th entry in which
% gives the number of regularites (spectral power) in the i-1-th degree in the 
% power series for the concept. 
%
% The "algebraic complexity" of a concept is the total amount of spectral
% power, with the power at each degree weighed by a weighting function
% determined by (global) weighting_style (default: degree plus one, 'K+1'). 
%
% Defaults:
%     If power_series is run without duality or feature_grid being set,
%     they will be set to defaults:
%        duality:         ruleout
%        feature_grid:    min necessary to accomodate the concept, >= 2.
%        weighting_style: K+1
%
global feature_grid;   
global duality;  % 'ruleout' or 'rulein'
global current_concept;
%----------------------------------------------
current_concept = [];
%
switch nargin
    case 0
        feature_grid = [2 2 2];         % Default feature_grid; used in complexity_tool
        duality = 'ruleout';    
    case 1
        feature_grid = desired_feature_grid;
        duality = 'ruleout';
    case 2
        feature_grid = desired_feature_grid;
        duality = desired_duality;
end;
make_feature_grid(feature_grid);        % installs regularity_sequence; can be time-consuming.
